<?php declare(strict_types=1);

namespace USKit\Base\Parts;

use Error;
use Exception;
use Infomaker\Everyware\Base\OpenContent\OpenContentProvider;
use USKit\Base\Parts\NewsMLArticle;
use USKit\Base\ViewModels\Teaser;
use Infomaker\Everyware\Base\Interfaces\NewsItemPresentation;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;
use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Everyware\Support\Storage\Cache;
use Infomaker\Everyware\Twig\View;
use Everyware\Plugin\SettingsParameters\SettingsParameter;

class ArticleBodyPresentation implements NewsItemPresentation
{
    const FB_API_APP_ID_KEY = 'FB_API_APP_ID_KEY';
    const FB_API_ACCESS_TOKEN_KEY = 'FB_API_ACCESS_TOKEN_KEY';
    const FB_API_200_CACHE_DURATION = 'FB_API_200_CACHE_DURATION';
    const GOOGLE_MAPS_EMBED_API_KEY = 'GOOGLE_MAPS_EMBED_API_KEY';

    private $relatedArticles;

    public function __construct(PropertyObject $article)
    {
        $this->relatedArticles = [];

        foreach ($article->get('articles', []) as $oc_article) {
            if ($oc_article instanceof PropertyObject) {
                $this->relatedArticles[$oc_article->get('uuid')] = $oc_article->get('permalink');
            }
        }
    }

    protected function element(Item $item): string
    {

        $content = $this->isListElement($item) ? $this->generateListContent($item) : $item->get('content');

        return $this->generateArticlePart('element', [
            'type' => $item->getType(),
            'content' => $content,
            'attributes' => $item->toArray()
        ]);
    }

    private function image(ImageItem $item): string
    {
        $image = new Image($item);
        return $this->generateArticlePart('image', $image->getViewData());
    }

    private function gallery_image(ImageItem $item): string
    {
        $image = new Image($item);
        return $this->generateArticlePart('gallery_image', $image->getViewData());
    }    

    private function table(Item $item): string
    {
        return $this->generateArticlePart('table', $item->toArray());
    }

    private function htmlEmbed(Item $item)
    {
        return $item->get('content');
    }

    private function imageGallery(Item $item): string
    {
        $imageGallery_Ads_Switch = SettingsParameter::getValue('gallery_image_ad_switch');
        $imageGallery_Ad_ID = SettingsParameter::getValue('gallery_image_ad_id');
        $imageGallery_Ad_Frequency = SettingsParameter::getValue('gallery_image_ad_frequency');

        if ( $imageGallery_Ads_Switch == 'true' ) {
            $images = array_map([&$this, 'gallery_image'], (array)$item->get('images'));

            $item->set('images', $images);
            $items = $item->toArray();

            $items['ad_id'] = $imageGallery_Ad_ID;
            $items['ad_frequency'] = $imageGallery_Ad_Frequency;

            return $this->generateArticlePart('gallery_with_ads', $items);
        } else {
            $images = array_map([&$this, 'gallery_image'], (array)$item->get('images'));

            $item->set('images', $images);

            return $this->generateArticlePart('gallery', $item->toArray());            
        }
    }

    private function contentPart(Item $item): string
    {
        $contentPartText = array_map([&$this, 'element'], (array)$item->get('text'));

        $item->set('text', implode('', $contentPartText));

        $item->set('caption', $item['el']->data->caption);// setting the caption for factbox

        return $this->generateArticlePart('content-part', $item->toArray());
    }

    private function relatedArticle(Item $item): string
    {
        if (isset($this->relatedArticles[$item->uuid])) {

            $articleQuery = OpenContentProvider::setup( [
                'q'                      => 'uuid:' . $item->uuid,
                'contenttypes'           => [ 'Article' ],
                'limit'                  => 1
            ] );

            $articleQuery->setPropertyMap( 'Article' );

            $articleData = array_map( function ( NewsMLArticle $article ) {
                $teaser = new Teaser( $article );
                return $teaser->getViewData();
            }, $articleQuery->queryWithRequirements() );

            if (count($articleData) > 0) {
                $item->fill($articleData[0]);
            }

            $item->set('permalink', $this->relatedArticles[$item->uuid]);

            return $this->generateArticlePart('related-article', $item->toArray());
        }
        if (isset($item['properties']['uuid'])) {
            $articleuuid = $item['properties']['uuid'];
            $articleQuery = OpenContentProvider::setup( [
                'q'                      => 'uuid:' . $articleuuid,
                'contenttypes'           => [ 'Article' ],
                'limit'                  => 1
            ] );

            $articleQuery->setPropertyMap( 'Article' );

            $articleData = array_map( function ( NewsMLArticle $article ) {
                $teaser = new Teaser( $article );
                return $teaser->getViewData();
            }, $articleQuery->queryWithRequirements() );

            if (count($articleData) > 0) {
                $item->fill($articleData[0]);
            }
            
            return $this->generateArticlePart('related-article', $item->toArray());
        }

        return '';
    }

    private function pdf(Item $item): string
    {
        return $this->generateArticlePart('pdf', [
            'url' => $item->get('url'),
            'title' => $item->get('title'),
        ]);
    }

    private function youtube(Item $item): string
    {
        return $this->generateArticlePart('youtube', $item->toArray());
    }

    private function review(Item $item): string
    {
        $text = array_map([&$this, 'element'], (array)$item->get('text'));

        $item->set('text', implode('', $text));

        return $this->generateArticlePart('review', $item->toArray());
    }

    private function youplay(Item $item): string
    {
        return $this->generateArticlePart('youplay', $item->toArray());
    }

    private function socialEmbed(Item $item): ?string
    {
        $links = $item->get('content');

        if (empty($links)) {
            return '';
        }

        if ( ! ($linkItem = array_shift($links)) instanceof Item) {
            return '';
        }
        switch ($linkItem->getType()) {
            case 'x-im/tweet':
                return $this->twitter($linkItem);
                break;
            case 'x-im/facebook-post':
                return $this->facebook($linkItem);
                break;
            case 'x-im/instagram':
                return $this->instagram($linkItem);
                break;
            default:
                return '';
                break;
        }
    }

    private function instagram(Item $item): string
    {
        $embed_url = $item->embed_url ?? $item->url ?? '';
        $key = md5("instagram_{$embed_url}");
        $embedHtml = Cache::get($key) ?: '';


        $appId = SettingsParameter::getValue(self::FB_API_APP_ID_KEY) ?? '';
        $accessToken = SettingsParameter::getValue(self::FB_API_ACCESS_TOKEN_KEY) ?? '';

        $timeToCache200 = SettingsParameter::getValue(self::FB_API_200_CACHE_DURATION) ?? 60;

        if (empty($embedHtml)) {
            try {
                if (isset($item->url)) {
                    $apiResponse = wp_remote_get('https://graph.facebook.com/v8.0/instagram_oembed?url=' . $item->url . '&access_token=' . $appId . '|' . $accessToken);
                    if($apiResponse['response']['code'] === 200){
                        $embedHtml = json_decode($apiResponse['body'], false)->html;
                        Cache::set($key, $embedHtml, $timeToCache200 * MINUTE_IN_SECONDS);
                    } else {
                        Cache::set($key, $embedHtml, 1 * MINUTE_IN_SECONDS);
                    }
                }
            } catch (Error $e) {
                return '';
            } catch (Exception $e) {
                return '';
            }
        }

        return $this->generateArticlePart('instagram', [
            'content' => $embedHtml
        ]);
    }

    private function facebook(Item $item): string
    {
        return $this->generateArticlePart('facebook', [
            'url' => $item->url ?? ''
        ]);
    }

    private function twitter(Item $item): string
    {
        return $this->generateArticlePart('twitter', [
            'url' => $item->url ?? ''
        ]);
    }

    private function google_maps_embed(Item $item): string
    {
        return $this->generateArticlePart('google_maps_embed', [
            'api_key'   => SettingsParameter::getValue(self::GOOGLE_MAPS_EMBED_API_KEY) ?? '',
            'zoom'      => $item->zoom,
            'latitude'  => substr(explode(' ', $item->geometry)[1], 1),
            'longitude' => substr(explode(' ', $item->geometry)[2], 0, -1)
        ]);
    }

    private function generateListContent(Item $item): string
    {
        if ( ! $this->isListElement($item)) {
            return '';
        }

        $listItems = str_replace('list-item', 'li', $item->getFilteredContent(['list-item']));

        switch ($item->getType()) {
            case 'x-im/unordered-list':
                return "<ul>{$listItems}</ul>";
            case 'x-im/ordered-list':
                return "<ol>{$listItems}</ol>";
        }

        return '';
    }

    private function isListElement(Item $item)
    {
        $validListTypes = ['x-im/unordered-list', 'x-im/ordered-list'];

        return in_array($item->getType(), $validListTypes, true);
    }

    private function generateArticlePart($part, $data): string
    {
        $data['imengine_path'] = get_theme_mod('image_endpoint_url') ?? null;//setting the image path for factbox
        
        try {
            return View::generate("@base/article/{$part}", $data);
        } catch (\Twig_Error $e) {
            NewRelicLog::error('Failed to generate article body part', $e);
        }

        return '';
    }

    public function generateItem(string $type, Item $item): string
    {
        switch ($type) {
            case 'x-im/youtube':
                return $this->youtube($item);
            case 'x-im/youplay':
                return $this->youplay($item);
            case 'x-im/image':
                return $item instanceof ImageItem ? $this->image($item) : '';
            case 'x-im/link':
                return implode('', $item->toArray());
            case 'x-im/article':
                return $this->relatedArticle($item);
            case 'x-gm/review':
                return $this->review($item);
            case 'x-im/socialembed':
                return $this->socialEmbed($item);
            case 'x-im/content-part':
                return $this->contentPart($item);
            case 'x-im/htmlembed':
                return $this->htmlEmbed($item);
            case 'x-im/imagegallery':
                return $this->imageGallery($item);
            case 'x-im/table':
                return $this->table($item);
            case 'x-im/pdf':
                return $this->pdf($item);
            case 'x-im/mapembed';
                return $this->google_maps_embed($item);
            default:
                return $this->element($item);

        }
    }
}
